/***************************************************************************************************
*                           Sample for the usage of class Episodic_TopoART                         *
*                           Created by Marko Tscherepanow, 20 October 2012                         *
***************************************************************************************************/

// Compile: mcs /r:LibTopoART.dll Episodic_TopoART_sample1.cs
// Run: mono Episodic_TopoART_sample1.exe

using System;
using System.IO;
using System.Text.RegularExpressions;
using LibTopoART;
using System.Globalization;

namespace LibTopoART_samples
{
	/// <summary>
	/// Sample using artificial two-dimensional data.
	/// <para>
	/// Like in "Marko Tscherepanow, Sina Kühnel, and Sören Riechers, 'Episodic  Clustering of Data Streams Using 
	/// a Topology-Learning Neural Network', European Conference on Artificial Intelligence (ECAI), Workshop on 
	/// Active and Incremental Learning (AIL), pp. 24-29, 2012." an Episodic TopoART network is trained with the 
	/// well-known Two Spirals dataset. Due to the incorporation of temporal information during learning, Episodic 
	/// TopoART is capable of creating two clusters each representing one spiral in an unsupervised way. These 
	/// clusters are formed by the nodes of module b (ETA b).
	/// </para>
	/// <para>The resulting network can be visualised using the R script <c>ShowEpisodicTopoARTResults.R</c> provided 
	/// in the subfolder <c>R</c>.
	/// </para>
	/// </summary>
	class Episodic_TopoART_sample1
	{
	    static void Main() {
			// Dataset
			string Dataset="data/TwoSpirals_dataset.txt";		
			long SampleNumber=194;
			long InputDimension=2;
			StreamReader DatasetFile;
			decimal[][] InputArray=new decimal[SampleNumber][];
			long i;
			string Line;
			DateTime start,end;
			TimeSpan time;
			Episodic_TopoART eta;

			// Start time measuring
			start=DateTime.Now;
			
			// Load dataset
			using(DatasetFile=new StreamReader(File.Open(Dataset, FileMode.Open))) {
				for(i=0;i<SampleNumber;i++) {
					Line=DatasetFile.ReadLine();
					string[] numbers = Regex.Split(Line, @"\s+");
					InputArray[i]=new decimal[InputDimension]; 
					InputArray[i][0]=Decimal.Parse(numbers[1],NumberStyles.Float,System.Globalization.CultureInfo.InvariantCulture);
					InputArray[i][1]=Decimal.Parse(numbers[2],NumberStyles.Float,System.Globalization.CultureInfo.InvariantCulture);
					InputArray[i][0]=(InputArray[i][0]+7.0m)/14m;
					InputArray[i][1]=(InputArray[i][1]+7.0m)/14m;
				}
			}
			
			// Initialise a Episodic TopoART network with appropriate parameter values (taken from the paper
			// given above)
			eta=new Episodic_TopoART(InputDimension,2,0.69m,200);
			eta.Beta_sbm=0.0m;
			eta.Phi=1;
			eta.Tau=200;
					
			// Train network
			for(i=0;i<SampleNumber;i++) 
				eta.Learn(InputArray[i]);
			Console.WriteLine("");
			
			// Determine clusters
			eta.ComputeClusterIDs();
			
			// Save network in human readable form
			eta.SaveText("Episodic_TopoART_TwoSpirals_dataset.txt");

			// Stop time measuring
			end=DateTime.Now;

			// Output the required time
			time=end-start;
			Console.WriteLine("time for computation: "+time);
		}
	}
}