/***************************************************************************************************
*                                Sample for the usage of class TopoART                             *
*                           Created by Marko Tscherepanow, 30 December 2012                        *
***************************************************************************************************/

// Compile: mcs /r:LibTopoART.dll TopoART_sample2.cs
// Run: mono TopoART_sample2.exe

using System;
using System.IO;
using System.Text.RegularExpressions;
using LibTopoART;
using System.Globalization;

namespace LibTopoART_samples
{
	/// <summary>
	/// Sample using artificial two-dimensional data.
	/// <para>
	/// Train TopoART with a two-dimensional dataset similar to the one used in "Marko Tscherepanow and Sören Riechers,
	/// 'An Incremental On-line Classifier for Imbalanced, Incomplete, and Noisy Data', European Conference on 
	/// Artificial Intelligence (ECAI), Workshop on Active and Incremental Learning (AIL), pp. 18-23, 2012." This 
	/// dataset comprises six clusters (each containing 15,000 samples) as well as 10,000 noise samples. These 
	/// samples were mixed randomly.
	/// </para>
	/// <para>The resulting network can be visualised using the R script <c>ShowTopoARTResults.R</c> provided 
	/// in the subfolder <c>R</c>.
	/// </para>
	/// </summary>
	class TopoART_sample2
	{
	    static void Main() {
			// Dataset
			string Dataset="data/AIL12-like_dataset.txt";		
			long SampleNumber=100000;
			long InputDimension=2;
			StreamReader DatasetFile;
			decimal[][] InputArray=new decimal[SampleNumber][];
			long i;
			string Line;
			DateTime start,end;
			TimeSpan time;
			ITopoART t;

			// Choose TopoART implementation (false=TopoART;true=Fast_TopoART). If true, class Fast_TopoART will be used 
			// rather than class TopoART. As a result, the computation is accelerated but less accurate. From
			// an algorithmic point of view, both implementations are equal. But Fast_TopoART uses long rather than decimal
			// for the internal represention of data.
			bool USE_FAST_TOPOART=false;

			// Start time measuring
			start=DateTime.Now;
			
			// Load dataset
			using(DatasetFile=new StreamReader(File.Open(Dataset, FileMode.Open))) {
				for(i=0;i<SampleNumber;i++) {
					Line=DatasetFile.ReadLine();
					string[] numbers = Regex.Split(Line, @"\s+");
					InputArray[i]=new decimal[InputDimension]; 
					InputArray[i][0]=Decimal.Parse(numbers[1],NumberStyles.Float,System.Globalization.CultureInfo.InvariantCulture);
					InputArray[i][1]=Decimal.Parse(numbers[2],NumberStyles.Float,System.Globalization.CultureInfo.InvariantCulture);
				}
			}
			
			// Initialise a TopoART network with appropriate parameter values
			if(USE_FAST_TOPOART) t=new Fast_TopoART(InputDimension,2,0.92m);	
			else t=new TopoART(InputDimension,2,0.92m);
			t.Beta_sbm=0.6m;
			t.Phi=5;
			t.Tau=200;
					
			// Train network
			for(i=0;i<SampleNumber;i++) {
				t.Learn(InputArray[i]);
				if(i%1000==0) Console.Write(".");
			}
			Console.WriteLine("");
			
			// Determine clusters
			t.ComputeClusterIDs();
			
			// Save network in human readable form
			t.SaveText("TopoART_AIL12-like_dataset.txt");

			// Stop time measuring
			end=DateTime.Now;

			// Output the required time
			time=end-start;
			Console.WriteLine("time for computation: "+time);
		}
	}
}